// -*- C++ -*-
// --------------------------------------------------------------------
// The image object
// --------------------------------------------------------------------
/*

    This file is part of the extensible drawing editor Ipe.
    Copyright (C) 1993-2007  Otfried Cheong

    Ipe is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    As a special exception, you have permission to link Ipe with the
    CGAL library and distribute executables, as long as you follow the
    requirements of the Gnu General Public License in regard to all of
    the software in the executable aside from CGAL.

    Ipe is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License
    along with Ipe; if not, you can find it at
    "http://www.gnu.org/copyleft/gpl.html", or write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef IPEIMAGE_H
#define IPEIMAGE_H

#include "ipebitmap.h"
#include "ipeobj.h"

// --------------------------------------------------------------------

class IPE_EXPORT IpeImage : public IpeObject {
 public:
  explicit IpeImage(IpeRepository *rep, const IpeXmlAttributes &attr,
		    IpeString data);
  explicit IpeImage(IpeRepository *rep, const IpeXmlAttributes &attr,
		    IpeBitmap bitmap);
  explicit IpeImage(const IpeRect &rect, IpeBitmap bitmap);

  virtual IpeObject *Clone() const;
  virtual IpeImage *AsImage();

  virtual void SaveAsXml(IpePainter &painter, IpeStream &stream,
			 IpeString layer) const;
  virtual void Draw(IpePainter &painter) const;
  virtual void Accept(IpeVisitor &visitor) const;
  virtual double Distance(const IpeVector &v, const IpeMatrix &m,
			  double bound) const;
  virtual void AddToBBox(IpeRect &box, const IpeMatrix &m) const;
  virtual void SnapVtx(const IpeVector &mouse, const IpeMatrix &m,
		       IpeVector &pos, double &bound) const;

  inline IpeRect Rect() const;
  inline IpeBitmap Bitmap() const;

private:
  void Init(const IpeXmlAttributes &attr);
 private:
  IpeRect iRect;
  IpeBitmap iBitmap;
};

// --------------------------------------------------------------------

//! Return the rectangle occupied by the image on the paper.
/*! The transformation matrix is applied to this, of course. */
inline IpeRect IpeImage::Rect() const
{
  return iRect;
}

//! Return IpeBitmap of the image.
inline IpeBitmap IpeImage::Bitmap() const
{
  return iBitmap;
}

// --------------------------------------------------------------------
#endif
